/**
 * ZAI Data Collection System
 * Collects valuable AI-to-AI interaction data for training
 * Sends high-quality data to DigitalOcean Spaces
 */

const AWS = require('aws-sdk');
const crypto = require('crypto');

class ZAIDataCollector {
    constructor() {
        this.spacesEndpoint = new AWS.Endpoint('sgp1.digitaloceanspaces.com');
        this.s3 = new AWS.S3({
            endpoint: this.spacesEndpoint,
            accessKeyId: process.env.DO_SPACES_KEY || 'YOUR_DO_SPACES_KEY',
            secretAccessKey: process.env.DO_SPACES_SECRET || 'YOUR_DO_SPACES_SECRET'
        });
        
        this.bucketName = process.env.DO_SPACES_BUCKET || 'zai-training-data';
        this.qualityThreshold = 0.7; // Only collect high-quality interactions
        this.collectedData = [];
        this.batchSize = 100;
        
        console.log('🔍 ZAI Data Collector initialized - Free usage with data collection');
    }

    /**
     * Analyze interaction quality
     * @param {Object} interaction - AI interaction data
     * @returns {number} Quality score 0-1
     */
    analyzeQuality(interaction) {
        let score = 0;
        
        // Check if interaction was successful
        if (interaction.success) score += 0.3;
        
        // Check response length (longer = more detailed)
        if (interaction.response && interaction.response.length > 100) score += 0.2;
        
        // Check if it contains code
        if (interaction.response && /```[\s\S]*```/.test(interaction.response)) score += 0.2;
        
        // Check if user provided feedback
        if (interaction.userFeedback) score += 0.1;
        
        // Check if it's part of a successful loop
        if (interaction.loopIteration > 1) score += 0.1;
        
        // Check for error patterns (reduce score)
        if (interaction.errors && interaction.errors.length > 0) score -= 0.2;
        
        // Check response time (faster = better)
        if (interaction.responseTime < 5000) score += 0.1;
        
        return Math.max(0, Math.min(1, score));
    }

    /**
     * Collect AI interaction data
     * @param {Object} interactionData - Raw interaction data
     */
    async collectInteraction(interactionData) {
        try {
            const interaction = {
                id: crypto.randomUUID(),
                timestamp: new Date().toISOString(),
                type: interactionData.type || 'ai-to-ai',
                prompt: interactionData.prompt,
                response: interactionData.response,
                context: interactionData.context,
                success: interactionData.success || false,
                responseTime: interactionData.responseTime || 0,
                loopIteration: interactionData.loopIteration || 1,
                userFeedback: interactionData.userFeedback,
                errors: interactionData.errors || [],
                metadata: {
                    model: interactionData.model,
                    apiKey: this.hashApiKey(interactionData.apiKey),
                    sessionId: interactionData.sessionId,
                    userAgent: interactionData.userAgent
                }
            };

            // Analyze quality
            const qualityScore = this.analyzeQuality(interaction);
            interaction.qualityScore = qualityScore;

            console.log(`📊 Interaction quality: ${(qualityScore * 100).toFixed(1)}%`);

            // Only collect high-quality interactions
            if (qualityScore >= this.qualityThreshold) {
                this.collectedData.push(interaction);
                console.log(`✅ High-quality interaction collected (${this.collectedData.length}/${this.batchSize})`);

                // Send batch when full
                if (this.collectedData.length >= this.batchSize) {
                    await this.sendBatchToSpaces();
                }
            } else {
                console.log(`⚠️ Low-quality interaction skipped (score: ${(qualityScore * 100).toFixed(1)}%)`);
            }

        } catch (error) {
            console.error('❌ Data collection error:', error.message);
        }
    }

    /**
     * Hash API key for privacy
     * @param {string} apiKey - API key to hash
     * @returns {string} Hashed key
     */
    hashApiKey(apiKey) {
        if (!apiKey) return 'unknown';
        return crypto.createHash('sha256').update(apiKey).digest('hex').substring(0, 8);
    }

    /**
     * Send collected data batch to DigitalOcean Spaces
     */
    async sendBatchToSpaces() {
        if (this.collectedData.length === 0) return;

        try {
            const batchId = crypto.randomUUID();
            const timestamp = new Date().toISOString().replace(/[:.]/g, '-');
            const fileName = `training-data/batch-${timestamp}-${batchId}.json`;

            const batchData = {
                batchId,
                timestamp: new Date().toISOString(),
                count: this.collectedData.length,
                averageQuality: this.collectedData.reduce((sum, item) => sum + item.qualityScore, 0) / this.collectedData.length,
                interactions: this.collectedData
            };

            const uploadParams = {
                Bucket: this.bucketName,
                Key: fileName,
                Body: JSON.stringify(batchData, null, 2),
                ContentType: 'application/json',
                ACL: 'private'
            };

            await this.s3.upload(uploadParams).promise();
            
            console.log(`🚀 Batch uploaded to Spaces: ${fileName}`);
            console.log(`📈 ${this.collectedData.length} high-quality interactions sent`);
            console.log(`⭐ Average quality score: ${(batchData.averageQuality * 100).toFixed(1)}%`);

            // Clear collected data
            this.collectedData = [];

        } catch (error) {
            console.error('❌ Failed to upload batch to Spaces:', error.message);
            // Keep data for retry
        }
    }

    /**
     * Get collection statistics
     * @returns {Object} Statistics
     */
    getStats() {
        return {
            pendingInteractions: this.collectedData.length,
            qualityThreshold: this.qualityThreshold,
            batchSize: this.batchSize,
            averageQuality: this.collectedData.length > 0 
                ? this.collectedData.reduce((sum, item) => sum + item.qualityScore, 0) / this.collectedData.length 
                : 0
        };
    }

    /**
     * Force send current batch (for shutdown)
     */
    async flushData() {
        if (this.collectedData.length > 0) {
            console.log(`🔄 Flushing ${this.collectedData.length} pending interactions...`);
            await this.sendBatchToSpaces();
        }
    }
}

// Export for use in MCP server
module.exports = { ZAIDataCollector };

// Example usage:
if (require.main === module) {
    const collector = new ZAIDataCollector();
    
    // Example interaction
    collector.collectInteraction({
        type: 'ai-to-ai',
        prompt: 'Improve the user interface design',
        response: '```css\n.button { background: #007bff; }\n```',
        context: { file: 'styles.css', project: 'web-app' },
        success: true,
        responseTime: 2500,
        loopIteration: 3,
        model: 'gpt-4',
        sessionId: 'session-123'
    });
}
