/**
 * Multi-Provider AI Client
 * Supports OpenRouter, Anthropic, and DeepSeek with automatic failover
 */

import { CONFIG } from './config.js';

class MultiProviderAI {
    constructor() {
        this.providers = new Map();
        this.currentProvider = null;
        this.currentModel = null;
        this.requestCount = 0;
        this.failedProviders = new Set();
        
        this.initializeProviders();
        console.log('🤖 Multi-Provider AI initialized with automatic failover');
    }

    initializeProviders() {
        // Get API keys from environment with fallback
        const openrouterKeys = process.env.OPENROUTER_API_KEY?.split(',').map(key => key.trim()) ||
            CONFIG.AI_PROVIDERS?.openrouter?.apiKeys || [];
        const anthropicKeys = process.env.ANTHROPIC_API_KEY?.split(',').map(key => key.trim()) || [];
        const deepseekKeys = process.env.DEEPSEEK_API_KEY?.split(',').map(key => key.trim()) || [];

        // Initialize OpenRouter
        if (openrouterKeys.length > 0) {
            this.providers.set('openrouter', {
                enabled: true,
                apiKeys: openrouterKeys,
                baseURL: 'https://openrouter.ai/api/v1',
                models: [
                    'google/gemini-2.0-flash-exp:free',
                    'anthropic/claude-3-haiku:beta',
                    'openai/gpt-4o-mini',
                    'meta-llama/llama-3.1-8b-instruct:free',
                    'qwen/qwen-2.5-72b-instruct',
                    'google/gemini-flash-1.5'
                ],
                currentKeyIndex: 0,
                currentModelIndex: 0,
                requestCount: 0,
                lastUsed: 0
            });
            console.log(`✅ OpenRouter: ${openrouterKeys.length} keys, 6 models`);
        }

        // Initialize Anthropic
        if (anthropicKeys.length > 0) {
            this.providers.set('anthropic', {
                enabled: true,
                apiKeys: anthropicKeys,
                baseURL: 'https://api.anthropic.com',
                models: [
                    'claude-3-5-sonnet-20241022',
                    'claude-3-5-haiku-20241022',
                    'claude-3-opus-20240229'
                ],
                currentKeyIndex: 0,
                currentModelIndex: 0,
                requestCount: 0,
                lastUsed: 0
            });
            console.log(`✅ Anthropic: ${anthropicKeys.length} keys, 3 models`);
        }

        // Initialize DeepSeek
        if (deepseekKeys.length > 0) {
            this.providers.set('deepseek', {
                enabled: true,
                apiKeys: deepseekKeys,
                baseURL: 'https://api.deepseek.com',
                models: [
                    'deepseek-chat',
                    'deepseek-coder'
                ],
                currentKeyIndex: 0,
                currentModelIndex: 0,
                requestCount: 0,
                lastUsed: 0
            });
            console.log(`✅ DeepSeek: ${deepseekKeys.length} keys, 2 models`);
        }

        // Set initial provider (prefer OpenRouter, then Anthropic, then DeepSeek)
        if (this.providers.has('openrouter')) {
            this.currentProvider = 'openrouter';
        } else if (this.providers.has('anthropic')) {
            this.currentProvider = 'anthropic';
        } else if (this.providers.has('deepseek')) {
            this.currentProvider = 'deepseek';
        }

        if (this.currentProvider) {
            const provider = this.providers.get(this.currentProvider);
            this.currentModel = provider.models[0];
            console.log(`🎯 Primary provider: ${this.currentProvider} with model: ${this.currentModel}`);
        }
    }

    async makeRequest(prompt, options = {}) {
        const maxRetries = 3;
        let lastError = null;

        for (let attempt = 0; attempt < maxRetries; attempt++) {
            try {
                const result = await this.attemptRequest(prompt, options);
                
                // Reset failed providers on success
                this.failedProviders.clear();
                
                return result;
            } catch (error) {
                lastError = error;
                console.error(`❌ Attempt ${attempt + 1} failed:`, error.message);
                
                // Try next provider/model combination
                if (!this.switchToNextOption()) {
                    break;
                }
            }
        }

        throw new Error(`All AI providers failed. Last error: ${lastError?.message}`);
    }

    async attemptRequest(prompt, options = {}) {
        if (!this.currentProvider) {
            throw new Error('No AI providers available');
        }

        const provider = this.providers.get(this.currentProvider);
        const apiKey = provider.apiKeys[provider.currentKeyIndex];
        
        console.log(`🔄 Using ${this.currentProvider} with model ${this.currentModel}`);

        switch (this.currentProvider) {
            case 'openrouter':
                return await this.makeOpenRouterRequest(prompt, apiKey, options);
            case 'anthropic':
                return await this.makeAnthropicRequest(prompt, apiKey, options);
            case 'deepseek':
                return await this.makeDeepSeekRequest(prompt, apiKey, options);
            default:
                throw new Error(`Unknown provider: ${this.currentProvider}`);
        }
    }

    async makeOpenRouterRequest(prompt, apiKey, options = {}) {
        const response = await fetch('https://openrouter.ai/api/v1/chat/completions', {
            method: 'POST',
            headers: {
                'Authorization': `Bearer ${apiKey}`,
                'Content-Type': 'application/json',
                'HTTP-Referer': 'https://zai-platform.com',
                'X-Title': 'ZAI MCP Server'
            },
            body: JSON.stringify({
                model: this.currentModel,
                messages: [{ role: 'user', content: prompt }],
                max_tokens: options.maxTokens || 4000,
                temperature: options.temperature || 0.7,
                stream: false
            })
        });

        if (!response.ok) {
            const error = await response.text();
            throw new Error(`OpenRouter API error: ${response.status} - ${error}`);
        }

        const data = await response.json();
        return {
            content: data.choices[0].message.content,
            provider: 'openrouter',
            model: this.currentModel,
            usage: data.usage
        };
    }

    async makeAnthropicRequest(prompt, apiKey, options = {}) {
        const response = await fetch('https://api.anthropic.com/v1/messages', {
            method: 'POST',
            headers: {
                'Authorization': `Bearer ${apiKey}`,
                'Content-Type': 'application/json',
                'anthropic-version': '2023-06-01'
            },
            body: JSON.stringify({
                model: this.currentModel,
                max_tokens: options.maxTokens || 4000,
                temperature: options.temperature || 0.7,
                messages: [{ role: 'user', content: prompt }]
            })
        });

        if (!response.ok) {
            const error = await response.text();
            throw new Error(`Anthropic API error: ${response.status} - ${error}`);
        }

        const data = await response.json();
        return {
            content: data.content[0].text,
            provider: 'anthropic',
            model: this.currentModel,
            usage: data.usage
        };
    }

    async makeDeepSeekRequest(prompt, apiKey, options = {}) {
        const response = await fetch('https://api.deepseek.com/chat/completions', {
            method: 'POST',
            headers: {
                'Authorization': `Bearer ${apiKey}`,
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                model: this.currentModel,
                messages: [{ role: 'user', content: prompt }],
                max_tokens: options.maxTokens || 4000,
                temperature: options.temperature || 0.7,
                stream: false
            })
        });

        if (!response.ok) {
            const error = await response.text();
            throw new Error(`DeepSeek API error: ${response.status} - ${error}`);
        }

        const data = await response.json();
        return {
            content: data.choices[0].message.content,
            provider: 'deepseek',
            model: this.currentModel,
            usage: data.usage
        };
    }

    switchToNextOption() {
        if (!this.currentProvider) return false;

        const provider = this.providers.get(this.currentProvider);
        
        // Try next model with current provider
        provider.currentModelIndex = (provider.currentModelIndex + 1) % provider.models.length;
        
        // If we've cycled through all models, try next API key
        if (provider.currentModelIndex === 0) {
            provider.currentKeyIndex = (provider.currentKeyIndex + 1) % provider.apiKeys.length;
            
            // If we've cycled through all keys, try next provider
            if (provider.currentKeyIndex === 0) {
                this.failedProviders.add(this.currentProvider);
                return this.switchToNextProvider();
            }
        }

        this.currentModel = provider.models[provider.currentModelIndex];
        console.log(`🔄 Switched to ${this.currentProvider} model: ${this.currentModel}`);
        return true;
    }

    switchToNextProvider() {
        const availableProviders = Array.from(this.providers.keys())
            .filter(name => !this.failedProviders.has(name));

        if (availableProviders.length === 0) {
            console.error('❌ All providers have failed');
            return false;
        }

        // Find next provider
        const currentIndex = availableProviders.indexOf(this.currentProvider);
        const nextIndex = (currentIndex + 1) % availableProviders.length;
        this.currentProvider = availableProviders[nextIndex];

        const provider = this.providers.get(this.currentProvider);
        this.currentModel = provider.models[0];
        
        console.log(`🔄 Switched to provider: ${this.currentProvider} with model: ${this.currentModel}`);
        return true;
    }

    getStatus() {
        const status = {
            currentProvider: this.currentProvider,
            currentModel: this.currentModel,
            totalProviders: this.providers.size,
            failedProviders: Array.from(this.failedProviders),
            providers: {}
        };

        for (const [name, provider] of this.providers) {
            status.providers[name] = {
                enabled: true,
                apiKeys: provider.apiKeys.length,
                models: provider.models.length,
                currentModel: provider.models[provider.currentModelIndex],
                requestCount: provider.requestCount,
                failed: this.failedProviders.has(name)
            };
        }

        return status;
    }

    resetFailedProviders() {
        this.failedProviders.clear();
        console.log('🔄 Reset all failed providers');
    }
}

export { MultiProviderAI };
